﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;

namespace Simple
{
    /* The COM method and properties interface */
    [Guid("1953ECF4-6036-4432-9A75-EFB334A7FBCD")]
    public interface SimpleCOMInterface
    {
        [DispId(1)]
        int Count { get; }

        [DispId(2)]
        void Reset();
    }

    /* The COM events interface */
    [Guid("90EE1AC1-1C05-4FAA-BD6C-66FF8D226AC2"),
    InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
    public interface SimpleCOMEvents
    {
        [DispId(3)]
        void Click ();
    }

    /* The COM visiable Simple COM class */
    [Guid("84893F0D-BCC3-47FD-9B7C-2D958330D2F0"),
    ClassInterface(ClassInterfaceType.AutoDual),
    ComSourceInterfaces(typeof(SimpleCOMEvents)),
    ProgId("SimpleCOM")]
    public class SimpleCOM : SimpleCOMInterface
    {
        /* The COM event that allows users of the COM object to subscribe to the
         * Simple App click event. */
        public delegate void ClickDel();
        public event ClickDel Click;

        /* Constructor which initializes the Simple App and subscribes to the
         * click event and display the Simple App. */
        public SimpleCOM()
        {
            _app = new SimpleApp();

            /* When the button is clicked in SimpleApp execute the _triggerCOMClickEvent method. */
            _app.comClick += new SimpleApp.ClickHandler(_triggerCOMClickEvent);

            _app.Show();
        }

        /* A public read-only property that allows users of the COM object to
         * see what the count is. */
        public int Count
        {
            get
            {
                return _app.Count;
            }
        }

        /* A public method that allows users of the COM object to reset the
         * Simple App to zero. */
        public void Reset()
        {
            _app.Reset();
        }

        /* The Simple App object. */
        private SimpleApp _app;

        /* When the Simple App button is pressed this method is called and we
         * trigger the COM event here so users of the COM object will get passed
         * on a click event. */
        private void _triggerCOMClickEvent (SimpleApp b, EventArgs e)
        {
            /* If the COM event is subscribed to trigger it. */
            if (Click != null)
                Click();
        }
    }
}
